<?php
/**
 * Linear Checkout for WooCommerce by Cartimize
 * Copyright (c) 2020 Revmakx LLC
 * revmakx.com
 */
namespace Cartimize\Actions;
use Cartimize\Core\ActionCore;

class RemoveCouponAction extends ActionCore {

	/**
	 * ApplyCouponAction constructor.
	 *
	 * @since 1.0.0
	 * @access public
	 * @param $id
	 */
	public function __construct( $id, $no_privilege, $action_prefix ) {
		parent::__construct( $id, $no_privilege, $action_prefix );
	}

	/**
	 * Applies the coupon discount and returns the new totals
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function action() {
		ob_start();
		$coupon = isset( $_POST['coupon_code'] ) ? wc_format_coupon_code( wp_unslash( $_POST['coupon_code'] ) ) : false; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

		if ( empty( $coupon ) ) {
			$msg = array( 'error' => esc_html__( 'Sorry there was a problem removing this coupon.', 'linear-checkout-for-woo-by-cartimize' ) );
			$message['error']['#coupon-notice-container'] = $msg['error'];
			$html = ob_get_clean();
		} else {
			WC()->cart->remove_coupon( $coupon );
			WC()->cart->calculate_totals();

			$discount_amounts = array();

			//  We set it true to here so when the HTML is generated the target is correct
			wc_maybe_define_constant( 'WOOCOMMERCE_CHECKOUT', true );
			$html = ob_get_clean();
			ob_start();
			cartimize_applied_coupon_html();
			$coupon_html = ob_get_contents();
			ob_clean();
			ob_end_clean();

			$response = array(
				'new_totals'    => array(
					'new_subtotal'       => WC()->cart->get_cart_subtotal(),
					'new_shipping_total' => WC()->cart->get_cart_shipping_total(),
					'new_taxes_total'    => WC()->cart->get_cart_tax(),
					'new_total'          => WC()->cart->get_total(),
				),
				'needs_payment' => WC()->cart->needs_payment(),
				'fees'          => $this->prep_fees(),
				'fragments' => array(
					'#cartimize-applied-coupon' => $coupon_html,
				), 

				

			);
			$response['code'] = sanitize_text_field( $_POST['coupon_code'] );
			$message = array();
		}

		$response[ 'notices' ] = $message;
		$response[ 'html' ] = $html;

		$this->out(
			$response
		);
	}

	function prep_fees() {
		$fees = [];

		foreach ( WC()->cart->get_fees() as $fee ) {
			$out         = (object) [];
			$out->name   = $fee->name;
			$out->amount = ( 'excl' == WC()->cart->tax_display_cart ) ? wc_price( $fee->total ) : wc_price( $fee->total + $fee->tax );
			$fees[]      = $out;
		}

		return $fees;
	}
}
